import { app, BrowserWindow, globalShortcut, Menu, Tray } from 'electron';
import path from 'node:path';
import started from 'electron-squirrel-startup';
import { initialize, preloadData } from './IpcMainHandler';

const isDevelopment: boolean = process.env.NODE_ENV === 'development';

console.log('[Main] API Host:', process.env.EIAC_DESKTOP_API_HOST);

// Ensure only one instance is running
const gotTheLock: boolean = app.requestSingleInstanceLock();
if (!gotTheLock) {
  app.exit(0); // 使用 exit 而不是 quit，确保立即退出
} else {
  app.on('second-instance', (event: Event, cmdArgs: string[], workingDirectory: string) => {
    // 当运行第二个实例时，我们应该聚焦到主窗口
    const win = BrowserWindow.getAllWindows()[0];
    if (win) {
      win.show(); // 确保窗口显示
      if (win.isMinimized()) win.restore();
      win.focus();
    }
  });
}

// Handle creating/removing shortcuts on Windows when installing/uninstalling.
if (started) {
  app.exit(0); // 使用 exit 而不是 quit，确保立即退出
}

const createWindow = () => {
  // Create the browser window.
  const win = new BrowserWindow({
    minWidth: 1024,
    minHeight: 768,
    autoHideMenuBar: true,
    center: true,
    //resizable: false,
    //minimizable: false,
    //maximizable: false,
    //alwaysOnTop: true,
    webPreferences: {
      contextIsolation: true, // 启用上下文隔离。
      nodeIntegration: false, // 禁用 Node.js 集成，提高安全性。
      preload: path.join(__dirname, 'preload.js'), // 预加载脚本
      webviewTag: true, // 启用webview标签
      //webSecurity: false, // 禁用web安全策略，允许跨域请求
      //allowRunningInsecureContent: true, // 允许运行不安全的内容
      //sandbox: false // 禁用沙箱
    },
  });

  // Hide top menu bar
  win.setMenuBarVisibility(false);
  win.setAutoHideMenuBar(true);
  win.setMenu(null);

  // // 配置 webview 的权限
  // win.webContents.session.webRequest.onBeforeSendHeaders((details, callback) => {
  //   callback({
  //     requestHeaders: {
  //       ...details.requestHeaders,
  //       'Origin': 'http://1.12.73.211:8848'
  //     }
  //   });
  // });

  // // 配置 webview 的响应头
  // win.webContents.session.webRequest.onHeadersReceived((details, callback) => {
  //   callback({
  //     responseHeaders: {
  //       ...details.responseHeaders,
  //       'Access-Control-Allow-Origin': ['*'],
  //       'Access-Control-Allow-Methods': ['GET, POST, PUT, DELETE, OPTIONS'],
  //       'Access-Control-Allow-Headers': ['Content-Type, Authorization']
  //     }
  //   });
  // });

  win.webContents.on('did-attach-webview', (event, webContents) => {
    webContents.setWindowOpenHandler((details) => {
      if (isDevelopment) {
        console.log('webview-new-window', webContents, details);
      }
      win.webContents.send('webview-new-window', webContents.id, details.url);
      return { action: 'deny' };
    });
  });

  // Set session
  win.webContents.session.setPermissionRequestHandler((webContents, permission, callback) => {
    callback(true);
  });

  // Minimize to system tray
  win.on('minimize', () => {
    win.hide();
  });

  // Listen window close (user clicks the X in the top right corner)
  win.on('close', (event) => {
    event.preventDefault();
    win.hide();
  });

  // production a bad jabe
  const handleDevTools = () => {
    if (win.webContents.isDevToolsOpened()) {
      win.webContents.closeDevTools();
    } else {
      win.webContents.openDevTools();
    }
  };
  globalShortcut.register("CommandOrControl+Shift+I", handleDevTools);
  if (isDevelopment) {
    win.webContents.openDevTools();
  }

  // and load the login.html of the app.
  if (MAIN_WINDOW_VITE_DEV_SERVER_URL) {
    win.loadURL(`${MAIN_WINDOW_VITE_DEV_SERVER_URL}/login.html`);
  } else {
    win.loadFile(path.join(__dirname, `../renderer/${MAIN_WINDOW_VITE_NAME}/login.html`));
  }

  // Create tray icon
  const contextMenu = Menu.buildFromTemplate([
    { label: '显示窗口', click: () => win.show() },
    { label: '退出程序', click: () => app.exit() }
  ]);
  const iconPath = isDevelopment ? path.join(__dirname, '../../assets/tray.png') : path.join(process.resourcesPath, 'assets', 'tray.png');
  const tray = new Tray(iconPath);
  tray.setToolTip('中国电信-工作台');
  tray.setContextMenu(contextMenu);

  // 点击托盘图标也可以恢复窗口（可选）
  tray.on('click', () => {
    win.show();
  });
};

// 在非 macOS 平台上，关闭所有窗口时退出应用
// Quit when all windows are closed, except on macOS. There, it's common
// for applications and their menu bar to stay active until the user quits
// explicitly with Cmd + Q.
app.on('window-all-closed', () => {
  if (process.platform !== 'darwin') {
    app.quit();
  }
});

// This method will be called when Electron has finished
// initialization and is ready to create browser windows.
// Some APIs can only be used after this event occurs.
app.whenReady().then(() => {
  preloadData();
  createWindow();

  // 在 macOS 平台上，点击 dock 图标时重新创建窗口
  app.on('activate', () => {
    // On OS X it's common to re-create a window in the app when the
    // dock icon is clicked and there are no other windows open.
    if (BrowserWindow.getAllWindows().length === 0) {
      createWindow();
    }
  });
});

initialize();
